
lrfRun = function(
  Dat = NULL,                  #  A data frame containing the numeric columns:
                               #    x: location coordinate,
                               #    z: concentration values
                               #  If no data is given, samples will be drawn from
                               #  the prior.
  OutFile  = "lrf",            #  Output file base name
  Bdy,                         #  Set from which mass pts are drawn; the results of a
                               #    call to the function "BBox".
                               #    The default is to take the range of x and
                               #    expand by 10%.
  ItnMax  = 1e4,               #  Maximum iterations after burn-in
  ItnBrn  = 1e3,               #  Burn-in iterations
  SavMax  = min(10,ItnMax),    #  Number of times to save state
  pX = c(pre=.10,bet=.10,add=.15,loc=.05,mag=.05,chsc=.10,rem=.15, spl=0.15, mer=0.15),
                               #  Move Prob's
                               #    pre: precision
                               #    beta: intercept
                               #    add: add an atom
                               #    gam: move an atom (change magnitude and location)
                               #    chsc: change the scale parameter of an atom
                               #    rem: remove an atom
                               #    spl: split an atom
                               #    mer: merge an atom
  dX = c(dnew=2.00,dmvmag=2.00,dmvsc=0.2,dmvloc=40.0, dspl = 1.00),
                               #  Step sizes for the move types.
                               #    dnew: scale of proposal for new atom
                               #    dmvmag: scale of step for a proposed change in atom
                               #      magnitude
                               #    dmvsc: scale of step for proposed change in atom variance
                               #    dmvloc: scale of step for a proposed change in atom
                               #      location
                               #    dspl: scale of step size for the atom magnitude during a
                               #      split move.
  PriPar  = list(apre = 1.418932,       #  Pre ~ Ga(apre,rpre) (the observation precision)
                 rpre = 1.103024,
                 asigma = 1,            #  Sigma_k^-2 ~ Ga(asigma,rsigma) (the kernel precision)
                 rsigma = 1,
                 mbet = 0.5, sbet=0.2, #  Beta  ~ No(mbet,sbet^2)
                 pM = rep(1/10, 10),            # The multinomial prior for |M|
                 agam = 0.0038404,  
                 rgam = 0.0736338),        #  LRF   ~ Levy( c SymmGa(gamma; agam, rgam) InvGa(sigma^2; asigma, rsigma) )
                                           #    LRF is the location and
                                           #    weight of one of the atoms (gamma_m)
  temps = 1.0,                 # the temperatures for the parallel tempering chain
  symm = T,                    # Whether the Levy random field should be symmetric
  printStats = T               # Whether to print summary statistics about the MCMC.
  )
{
#######################################################################
#
# Runs the MCMC, given a data set and bounding region.
#

  
  #   Check the inputs:
  if(any(pX<0)||sum(pX)==0) stop("lrfRun: Probs pX must be >=0");
  else if( length(pX) < 9 )  stop("lrfRun: pX vector must have length 9");
  else  names(pX) = c("pre", "bet", "add", "loc", "mag", "chsc", "rem", "spl", "mer");
  if(any(dX<0)) stop("lrfRun: Step size dX must be >=0");
  else if( length(dX) < 5 )  stop("lrfRun: dX vector must have length 5");
  else  names(dX) = c("dnew", "dmvmag", "dmvsc", "dmvloc", "dspl");

  if( missing(Bdy) || !inherits(Bdy,"Poly") ) {
    if( is.null( Dat ) ){
      xlim = c(0,1);
      Bdy = BBox( x = xlim );
    } else {
      Bdy = BBox(Dat$x, expa=0.10);
      xlim = Bdy@xlim;
    }
  } else {
    xlim = Bdy@xlim;
  }
  
  #   Save the function arguments and environment info
  MyId      = list();         #  General info about user & env.
  MyId$Call = deparse(match.call());  #  Get arguments from invocation
  MyId$Args = args(lrfRun)     #  Default values of other args
  MyId$Data = Dat;          #  Remember the input file
  MyId$ItnMax = ItnMax;          #  Iterations
  MyId$ItnBrn = ItnBrn;          #  Burn-in
  MyId$dX   = dX;              #  Remember the step sizes
  MyId$pX   = pX;              #  Remember the move prob.s
  MyId$temps = temps;

  #   Save information about the LARK prior and the number of data pts
  MyId$etc=list(
       N    = length(Dat$x),             # How many data points?
       bdy  = Bdy                           # Draw LRF from what polygon? 
    )
  MyId$Pri = PriPar;                        # Prior parameters
  
  #   Print this information to file
  OutFileLG = paste(OutFile,"Out.LG",sep="");   # Output Log (dput/dget)
  OutFileLGc = paste(OutFile,"Out.LGc",sep="");   # c-code Output Log
  OutFileFL = paste(OutFile,"Out.FL",sep="");   # Output, Fixed length.
                                              # This will contain the meta-data
                                              # for the output.  
  OutFileVL = paste(OutFile,"Out.VL",sep="");   # Output, Variable length
                                              # This will contain the samples.
  unlink(c("OutFileFL","OutFileVL","OutFileLG","OutFileLGc"));
  dput(MyId,OutFileLG);

  #   Run the MCMC, saving samples to files
  lrfMCMC.c( Dat = Dat, OutFileLG = OutFileLGc,
          OutFileFL = OutFileFL,
          OutFileVL = OutFileVL, xlim = xlim,
          ItnMax = ItnMax, ItnBrn = ItnBrn,
          SavMax = SavMax, 
          pX = pX, dX = dX, PriPar = PriPar,
          temps = temps,
          symm = symm,
          printStats = printStats );

  #   Return the call info
  invisible(MyId);
}


  
lrfMCMC.c = function( 
  Dat,                      #  A data frame containing the numeric columns:
                               #    x: location coordinates,
                               #    z: concentration values
  OutFileLG,                    #  The filename of the log file 
  OutFileFL,                    #  The filename of the fixed length file to be created
  OutFileVL,                    #  The filename of the variable length file to be created
  xlim,                        #  The boundaries of the set from which mass pts are drawn
  ItnMax,                      #  Maximum iterations
  ItnBrn,                      #  Burn-in iterations
  SavMax,                      #  Number of times to save state
  pX,                          #  Move Prob's
  dX,                          #  Step sizes
  PriPar,                      #  Hyperparameters for prior specification
  temps,                       #  Parallel tempering specification
  symm,                        #  Whether the random field should be symmetric
  printStats                   #  Whether to print summary statistics about the MCMC
  )
{
#######################################################################
#
# Internal use only.  Calls a C routine to runs the MCMC, given a data
#   set and bounding region.
#

  pMlen = length( PriPar@pM )
  if( is.null( Dat ) ){
    .C("MCMC",
       datX = as.double(c()),                datZ = as.double(c()),
       datLen = as.integer( 0 ),             OutLG = OutFileLG,
       OutFL = OutFileFL,                    OutVL = OutFileVL,
       xlim = as.double(xlim),       
       ItnMax = as.integer(ItnMax),          ItnBrn = as.integer(ItnBrn),
       SavMax = as.integer(SavMax),          pX = as.double(pX),
       dX = as.double(dX),                   apre = as.double(PriPar@apre),
       rpre = as.double(PriPar@rpre),        asigma = as.double(PriPar@asigma),
       rsigma = as.double(PriPar@rsigma),        
       mbet = as.double(PriPar@mbet),        sbet = as.double(PriPar@sbet),
       pM = as.double(PriPar@pM),            pMlen = as.integer( pMlen ),
       agam = as.double(PriPar@agam),        rgam = as.double(PriPar@rgam),
       temps = as.double(temps),             nTemps = as.integer( length(temps) ),
       symm = as.integer(symm),              printStats = as.integer(printStats),
       COPY = rep(FALSE,25) );
  } else {
    .C("MCMC",
       datX = as.double(Dat$x),              datZ = as.double(Dat$z),
       datLen = as.integer( length(Dat$x) ), OutLG = OutFileLG,
       OutFL = OutFileFL,                    OutVL = OutFileVL,
       xlim = as.double(xlim),               
       ItnMax = as.integer(ItnMax),          ItnBrn = as.integer(ItnBrn),
       SavMax = as.integer(SavMax),          pX = as.double(pX),
       dX = as.double(dX),                   apre = as.double(PriPar@apre),
       rpre = as.double(PriPar@rpre),        asigma = as.double(PriPar@asigma),
       rsigma = as.double(PriPar@rsigma),    
       mbet = as.double(PriPar@mbet),        sbet = as.double(PriPar@sbet),
       pM = as.double(PriPar@pM),            pMlen = as.integer( pMlen ),
       agam = as.double(PriPar@agam),        rgam = as.double(PriPar@rgam),
       temps = as.double(temps),             nTemps = as.integer( length(temps) ),
       symm = as.integer(symm),              printStats = as.integer(printStats),
       COPY = rep(FALSE,25));
  }
}

