#ifndef HARK_SAMP_H
#define HARK_SAMP_H


#include "tnt_array2d.h"
#include "tnt_array1d.h"
#include "tnt_array1d_utils.h"
#include "tnt_array2d_utils.h"
#include "moreArray1dUtils.h"
#include "randGen.h"
#include "timeKeeper.h"
#include <S.h>

using TNT::Array1D;
using TNT::Array2D;


class HarkSamp{
 private:
  Array1D<double> response_, XTy,
    unifProbs_, twoProbs_, 
    priorVar_,                // A vector that holds
			      // multiplication factors for the prior
			      // variances of the coefficient
			      // parameters.
    predPenalty_;
  Array2D<double> XTX;
  Array1D<int> nTry, nAcc;
  int nData_, nTemps_, nPredsTot, priorType_;
  char *rootFile_;
  double eta0hat, solveTime, yTy;
  timeKeeper tk;
  bool priorInMcmc_;

  /**************************************************************************
   **  get a single sample from a fixed-length file (a sample consists
   **  of nAtoms, SSE, pre, and beta)
   **/  
  int getSamp( FILE *FLfile, int *nAtoms, double *sse, double *pre,
	       double *beta );
  
  /**************************************************************************
   **  get the parameters of a single atom from a variable-length file 
   **/  
  int getAtom( FILE *VLfile, double *x, double *sig2, double *height);

  /**************************************************************************
   **  Open the files containing the 
   **  posterior samples for the separate LARK models.
   **/  
  Array1D<FILE *> openFiles( char *suffix );

  /**************************************************************************
   **  Open a file containing the 
   **  posterior samples for a LARK model.
   **/  
  FILE *openOneFile( int ind, char *suffix );

  /**************************************************************************
   **  Check that the files containing the 
   **  posterior samples for the separate LARK models all exist.
   **/  
  void checkFiles( char *suffix );

  /**************************************************************************
   **  Determine the number of posterior samples.
   **/  
  int getNiter();

  /**************************************************************************
   **  Sample from the conditional distribution of \phi^2 given \eta.
   **  See e.g. p.50 of Gelman, Carlin, Stern, and Rubin (2004).
   **/  
  void sampleResidVar
  ( const Array2D<double> &designMat,   // the design matrix
    const Array1D<double> &eta,         // the regression coefs
    double &phi2 );                     // for storing the new residual variance

  /**************************************************************************
   ** Sample from the conditional distribution of \eta given
   ** \phi^2. Computation is done as described in Gelman, Carlin,
   ** Stern, and Rubin (2004), Sec.14.2.
   **/  
  void sampleCoefs
  ( const Array2D<double> &designMat,   // the design matrix
    Array1D<double> &eta,               // for storing the regression coefs
    double phi2 );                     // the residual variance

  /**************************************************************************
   **  Calculate the least squares estimate of the regression
   **  coefficient vector.  Computations are done as described in
   **  Gelman, Carlin, Stern, and Rubin (2004), p.355-357.
   **/
  Array1D<double> LSE
  ( const Array2D<double> &designMat,   // the design matrix
    double &MSE );                      // Will be set to the MSE

  /**************************************************************************
   **  Calculate the posterior density of a Bayesian linear regression
   **  model with conjugate prior.  The prior is a normal inverse
   **  gamma distribution.  The parameters of the prior inverse gamma
   **  density for phi2 are given by priora and priorb.  The prior
   **  mean of the normal is taken to be zero, except for the
   **  intercept, which has prior mean eta0hat.  The prior precision
   **  matrix of the normal dist'n is specified by priorPrec.  The
   **  posterior density is also a normal inverse gamma, characterized
   **  by four quantities.  These are a, b: the params of the inverse
   **  gamma density; etaHat, Veta: the parameters of the normal
   **  density; note that the variance Veta should be multiplied by
   **  the value \phi^2 of the inverse-gamma-distributed variance
   **  parameter to get the posterior variance of eta.
   **/
  void bayesRegression
  ( const Array2D<double> &designMat,   // the design matrix
    const Array1D<int> &predsInd,       // the vector of inclusion indicators
    double &a,                          // Will be set to the posterior value
    double &b,
    Array1D<double> &etaHat,
    Array2D<double> &VetaInv,           // Will be set to the posterior
				        // precision matrix
    double priora,                      // priora,priorb: hyperparams for IG prior on phi2.
    double priorb,
    const Array2D<double> &priorPrec);  // After isolating the columns
                                        // and rows specified by
				        // predsInd, this is the prior
				        // precision matrix for the
				        // regression coefficients.

  /**************************************************************************
   **  Calculate the prior probability of the model given by predsInd,
   **  up to a normalizing constant.  Adjust according to predPenalty_.
   **/  
  double logModelPrior
  ( const Array1D<int> &predsInd );


  /**************************************************************************
   **  Update the model (predsInd) according to its marginal posterior probability
   **/  
  bool updatePredsInd
  ( const Array2D<double> &designMat,   // the design matrix
    Array1D<int> &predsInd,             // the model (to be updated)
    double &logMargProb,                // the value of the log marginal posterior
                                        // probability (to be updated)
    double priora,                      // priora,priorb: hyperparams for IG prior on phi2.
    double priorb,
    const Array2D<double> &priorPrec);  // After isolating the columns
				        // and rows specified by
                                        // predsInd, this is the prior
				        // precision matrix for the
				        // regression coefficients.

  /**************************************************************************
   **  Calculate the posterior probability of the model given by
   **  predsInd, up to a normalizing constant.  Computations done as
   **  described on p.349 of George and McCulloch (1997:Statistica
   **  Sinica).
   **/  
  double logModelProb
  ( const Array2D<double> &designMat,   // the design matrix
    const Array1D<int> &predsInd,
    double priora,                      // priora,priorb: hyperparams for IG prior on phi2.
    double priorb,
    const Array2D<double> &priorPrec );     // After isolating the columns
				      // and rows specified by
				      // predsInd, this is the prior
				      // precision matrix for the
				      // regression coefficients.

  /**************************************************************************
   **  Obtain a single sample from the posterior dist'n of the
   **  coefficients eta and the variance phi2 of a linear regression
   **  model, given the vector of inclusion indicators for the
   **  predictors.  The prior \pi( \phi^2 ) = IG( priora, priorb ) and
   **  \pi( \eta | \phi^2 ) = N( 0, \phi^2 priorPrec^{-1} ) is used,
   **  except that the prior mean for eta_0 is taken to be = eta0hat.
   **  Computations are done as described in Sudipto Banerjee's
   **  PUBH7440 course notes at www.biostat.umn.edu/~ph7440
   **/  
  void sampleCoefsAndVar
  ( const Array2D<double> &designMat,   // the design matrix
    Array1D<double> &eta,               // for storing the regression coefs
    double &phi2,                       // for storing the residual variance
    const Array1D<int> &predsInd,
    double priora,                      // priora,priorb: hyperparams for IG prior on phi2.
    double priorb,
    const Array2D<double> &priorPrec );     // After isolating the columns
				      // and rows specified by
				      // predsInd, this is the prior
				      // precision matrix for the
				      // regression coefficients.


  /**************************************************************************
   **  Perform several iterations of a Markov chain designed to sample
   **  from the posterior distribution of a regression model.
   **  response_ is used as the response vector.
   **/  
  void regressSamp
  ( const Array2D<double> &designMat,   // the design matrix
    Array1D<double> &eta,               // for storing the regression coefs
    double &phi2,                       // for storing the residual
					// variance
    Array1D<int> &predsInd,  	        // for storing the predictor indicators
    double &logMargProb,                // the value of the log marginal posterior
                                        // probability of the current
                                        // model.
    int nIter,                          // The number of iterations of the chain 
                                        // to run.
    double a,                           // a,b: hyperparams for IG prior on phi2.
    double b,
    const Array2D<double> &priorPrec);  // After isolating the columns
                                        // and rows specified by
				        // predsInd, this is the prior
				        // precision matrix for the
				        // regression coefficients.


  
  /**************************************************************************
   **  Read in the design matrices from file, for each posterior sample.
   **/  
  Array1D<Array2D<double> > getDesignMats
  ( int nIter );

  /* Calculate XTX and XTy, two quantities that are used often in the
     regression calculation */
  void updateXTX
  ( const Array2D<double> &designMat );

 public:
  HarkSamp(
     const Array1D<double> &response,    // the response vector
     char *rootFile,                  // the root name for the files
				      // containing the LARK MCMC
				      // samples.
     int nTemps,                         // The number of temps used in
				       // the original simulations.
     double *priorVar,                  // The vector of the entries of
				      // the diagonal prior covariance
				      // matrix for eta.
     int priorType,                   // priorType: 0= use the unit
				      // information prior of DiMatteo
				      // et al (2001; Biometrika).  In
				      // this case the priorVar
				      // argument is ignored.
				      // 1= use informative prior
				      // distributions; do not adjust
				      // for different variances of
				      // the predictor variables 
				      // when specifying the prior
				      // variance of the regression
				      // coefficients.  2= Do adjust.
     bool priorInMcmc,                // Whether or not to include the
                                      // model prior probabilities in 
                                      // the Markov chain (alternative 
                                      // is to use a uniform prior)
     const Array1D<double> &predPenalty);             
                                      // The fake prior odds for each
                                      // predictor indicator; the MCMC
                                      // is simulated with these prior odds
                                      // and they are removed for analysis
                                      // of the results.  Can be used to 
                                      // improve mixing.


  ~HarkSamp(){};

  /**************************************************************************
   **  Generate the samples from the linear regression posterior,
   **  given the samples from the LARK posterior distributions.
   **  Regression samples are saved to file.
   **/  
  void sample( char *outFile, const Array1D<int> &initPredsInd,
	       int nIter2ndStage);

};

#endif
