
#include <vector>
#include <fstream>
#include "LevyInfer.h"
#include "randGen.h"

#define SQR(x) ((x)*(x))
using TNT::Array1D;
using std::cout;
using std::endl;

LevyInfer::LevyInfer
( char *DatFL, char *DatVL, 
  const Array1D<double> &temps ){

  DatFL_ = DatFL;
  DatVL_ = DatVL;
  temps_ = temps.copy();
  nTemps = temps_.dim();
}

// Take a matrix that is represented as a vector of nRows Array1D objects, where 
// the length of each Array1D is the number of columns, and average the rows.
Array1D<double> rowAvg( 
  const std::vector< Array1D<double> > &mat
  ){

  Array1D<double> rowSum( mat[0].dim(), 0.0 );

  int i, nRows = mat.size();
  for(i=0; i<nRows; i++){
    rowSum = rowSum + mat[i];
  }
  return( rowSum / (double) nRows );
}

int getSamp( FILE *FLfile, int *nAtoms, double *sse, double *pre,
  double *beta ){

  double tempDbl[4];

  int nRead = fread( tempDbl, sizeof(double), 4, FLfile );
  if( nRead != 4 ){
    MESSAGE "Fixed-length file terminated unexpectedly" ERROR;
  }
  
  (*nAtoms) = (int) tempDbl[0];
  (*sse) = tempDbl[1];
  (*pre) = tempDbl[2];
  (*beta) = tempDbl[3];

  return(nRead);
}

int getAtom( FILE *VLfile, double *x, double *sig2, double *height){

  double tempDbl [3];

  int nRead = fread( tempDbl, sizeof(double), 3, VLfile );
  (*x) = tempDbl[0];
  (*sig2) = tempDbl[1];
  (*height) = tempDbl[2];

  return(nRead);
}

  /**************************************************************************
   **  Get the height of a kernel centered at x, with height=
   **  height and sig2 = sig2, at the specified locations.
   **/  
double LevyInfer::kernelHeight
( double xloc,
  double x, double sig2, double height ){

  Array1D<double> xlocs (1, xloc);  // create an array of length 1
  return( kernelHeight( xlocs, x, sig2, height )[0] );
}
					
  /**************************************************************************
   **  Get the height of a kernel centered at x, with height=
   **  height and sig2 = sig2, at the specified locations.
   **/  
Array1D<double> LevyInfer::kernelHeight(
  const Array1D<double> &xlocs, 
  double x, double sig2, double height ){

  Array1D<double> sqrDist, sqrDistTemp, kernelHeight;

  // calculate the squared distance of each location from the atom
  sqrDist = TNT::abs(xlocs - x);

  // calculate the height of the kernel at each location
  #ifdef ALT_KERN
    kernelHeight = sqrDist / sqrt(sig2);
    kernelHeight = kernelHeight + 1;
    kernelHeight = kernelHeight^(-4);
  #else
    sqrDist = sqrDist^2;
    kernelHeight = -0.5*sqrDist / sig2;
    kernelHeight = TNT::exp( kernelHeight );
  #endif
  
  kernelHeight = kernelHeight * height;

  return( kernelHeight );
}

Array1D<double> LevyInfer::meanVals( 
  const Array1D<double> &xlocs, 
  int tempInd, int sampNum
){

  if( sampNum >= 0 )
    return sampSurf( xlocs, sampNum, tempInd );

  int nLocs = xlocs.dim();
  Array1D<double> z( nLocs );

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, beta, x, sig2, height;
  int nAtoms, i, j;

  // Create a vector that is the sum of the surface height over the samples,
  // for each location.
  Array1D<double> surfHeightSum( nLocs, 0.0 );
  Array1D<double> sampSurfHeight( nLocs ), kernHeight;
  
  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];
  
  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    // Reset sampSurfHeight to beta at all locations.
    sampSurfHeight = beta;
    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      #ifdef DEBUG1
        std::cout << *tempDbl << " " << nAtoms << " " << sse << " " << pre << " " << beta << std::endl;
      #endif

      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
	if( j == tempInd ){
          kernHeight = kernelHeight( xlocs, x, sig2, height );
          sampSurfHeight = sampSurfHeight + kernHeight;
	}
      }
    }
    surfHeightSum = surfHeightSum + sampSurfHeight;
    nSamps++;
  }
  z = surfHeightSum / (double) nSamps;

  fclose(FLfile);
  fclose(VLfile);

  return(z);
}



Array1D<double> LevyInfer::stdevSurf
( const Array1D<double> &xlocs,
  int tempInd
){

  int nLocs = xlocs.dim();
  Array1D<double> z( nLocs );

  Array1D<double> meanSurf = meanVals( xlocs, tempInd ),
    squaredDiffSum( nLocs, 0.0 );

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, beta, x, sig2, height;
  int nAtoms, i, j;

  // Create a vector that is the sum of the surface height over the samples,
  // for each location.
  if( meanSurf.dim() != nLocs )
    MESSAGE "incorrect length of vector meanSurf" ERROR;

  Array1D<double> sampSurfHeight( nLocs ),
    kernHeight, sampDiff;
  
  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    // Reset sampSurfHeight to beta
    sampSurfHeight = beta;
    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      #ifdef DEBUG1
        std::cout << *tempDbl << " " << nAtoms << " " << sse << " " << pre << 
          " " << beta << std::endl;
      #endif

      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
	if( j == tempInd ){
          kernHeight = kernelHeight( xlocs, x, sig2, height );
          sampSurfHeight = sampSurfHeight + kernHeight;
	}
      }
    }
    sampDiff = meanSurf - sampSurfHeight;
    sampDiff = sampDiff * sampDiff;
    squaredDiffSum = squaredDiffSum + sampDiff;
    nSamps++;
  }
  z = squaredDiffSum / (double) nSamps;
  z = z^0.5;

  fclose(FLfile);
  fclose(VLfile);

  return(z);
}


Array1D<double> LevyInfer::percentAbove( 
  const Array1D<double> &xlocs, 
  double cutoff, int tempInd
){

  int nLocs = xlocs.dim();

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, beta, x, sig2, height;
  int nAtoms, i, j, k;

  Array1D<double> sampSurfHeight( nLocs ), kernHeight,
    nSampsAbove( nLocs, 0.0 );
  
  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    // Reset sampSurfHeight to beta
    sampSurfHeight = beta;
    for( k=0; k<nTemps; k++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      #ifdef DEBUG1
        std::cout << *tempDbl << " " << nAtoms << " " << sse << " " << pre << std::endl;
      #endif

      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
	if( k == tempInd ){
	  // if this is the desired temperature
          kernHeight = kernelHeight( xlocs, x, sig2, height );
          sampSurfHeight = sampSurfHeight + kernHeight;
	}
      }
    }
    for(j=0; j<nLocs; j++){
      if( sampSurfHeight[j] > cutoff )
        nSampsAbove[j] = nSampsAbove[j] + 1;
    }
    
    nSamps++;
  }

  fclose(FLfile);
  fclose(VLfile);

  return( nSampsAbove / (double) nSamps );
}

Array1D<double> LevyInfer::sampSurf( 
  const Array1D<double> &xlocs, 
  int sampNum, int tempInd
){

  int nLocs = xlocs.dim();
  Array1D<double> z( nLocs );

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, beta, x, sig2, height;
  int nAtoms, i, j;

  Array1D<double> sampSurfHeight( nLocs ), kernHeight;
  
  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    // Reset sampSurfHeight to beta
    sampSurfHeight = beta;
    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      #ifdef DEBUG1
        std::cout << *tempDbl << " " << nAtoms << " " << sse << " " << pre << std::endl;
      #endif

      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
	if( j == tempInd ){
	  // if this is the desired temperature
	  kernHeight = kernelHeight( xlocs, x, sig2, height );
          sampSurfHeight = sampSurfHeight + kernHeight;
	}
      }
    }
    if (nSamps == sampNum)
      return( sampSurfHeight );
    nSamps++;
  }

  fclose(FLfile);
  fclose(VLfile);

  return(sampSurfHeight);
}

/**************************************************************************
 **  Get the value of the posterior surface at a particular
 **  location, for each posterior sample.
 **/  
Array1D<double> LevyInfer::surfSamps( double xloc, int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> z;

  double sse, pre, beta, x, sig2, height, sampSurfHeight, kernHeight;
  int nAtoms, i, j;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];
  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    // Reset surface height
    sampSurfHeight = beta;
    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
	if( j == tempInd ){
          kernHeight = kernelHeight( xloc, x, sig2, height );
          sampSurfHeight = sampSurfHeight + kernHeight;
	}
      }
    }
    z.push_back(sampSurfHeight);
    nSamps++;
  }
  
  // Convert to a doubles array
  Array1D<double> zArr(nSamps);
  for(i=0; i<nSamps; i++)
    zArr[i] = z[i];

  fclose(FLfile);
  fclose(VLfile);

  return(zArr);
}


/**************************************************************************
 **  Get the sums of the atom heights for each iteration of the MCMC.
 **/  
Array1D<double> LevyInfer::zsums( int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> zsum;

  double sse, pre, beta, x, sig2, height, heightSum;
  int nAtoms, i, j;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  #ifdef DEBUG_ZSUMS
    cout << "No. temps: " << nTemps << "  Desired temp. index: " << tempInd << endl;
  #endif

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.
    #ifdef DEBUG_ZSUMS
      cout << "Iteration " << (*tempDbl) << ":" << endl;
    #endif

    // Reset jump height sum to zero
    heightSum = 0;
    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      #ifdef DEBUG_ZSUMS
        cout << "# atoms: " << nAtoms << "  pre: " << pre << endl;
      #endif

      for(i=0; i<nAtoms; i++){
	// for each atom
	// get the parameters of this atom from the variable-length file
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
  	    MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
        if( j == tempInd ){
	  // if this is the desired temperature
	  // contribute to the calculation of heightSum
          heightSum += fabs(height);
        }
      }
    }
    zsum.push_back(heightSum);
    nSamps++;
  }
  
  // Convert to a doubles array
  Array1D<double> zsumArr(nSamps);
  for(i=0; i<nSamps; i++)
    zsumArr[i] = zsum[i];
  #ifdef DEBUG_ZSUMS
    cout << "zsums: " << zsumArr << endl;
  #endif

  fclose(FLfile);
  fclose(VLfile);

  return(zsumArr);
}

/**************************************************************************
 **  Get the sums of the atom locations for each iteration of the MCMC.
 **/  
Array1D<double> LevyInfer::locSums( int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> locSum;

  double sse, pre, beta, x, sig2, height, cumuSum;
  int nAtoms, i, j;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    // Reset atom location sum to zero
    cumuSum = 0;
    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
        if( j == tempInd ){
	  // if this is the desired temperature
	  cumuSum += x;
	}
      }
    }
    locSum.push_back(cumuSum);
    nSamps++;
  }
  
  // Convert to a doubles array
  Array1D<double> sumArr(nSamps);
  for(i=0; i<nSamps; i++)
    sumArr[i] = locSum[i];

  fclose(FLfile);
  fclose(VLfile);

  return(sumArr);
}

/**************************************************************************
 **  Get the sums of the atom variances for each iteration of the MCMC.
 **/  
Array1D<double> LevyInfer::varSums( int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> varSum;

  double sse, pre, beta, x, sig2, height, cumuSum;
  int nAtoms, i, j;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    // Reset atom variance sum to zero
    cumuSum = 0;
    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
        if( j == tempInd ){
	  // if this is the desired temperature
	  cumuSum += sig2;
	}
      }
    }
    varSum.push_back(cumuSum);
    nSamps++;
  }
  
  // Convert to a doubles array
  Array1D<double> sumArr(nSamps);
  for(i=0; i<nSamps; i++)
    sumArr[i] = varSum[i];

  fclose(FLfile);
  fclose(VLfile);

  return(sumArr);
}

/**************************************************************************
 **  Get the average kernel variance (sigma^2) for each iteration of
 **  the MCMC.
 **/  
Array1D<double> LevyInfer::avgSig2( int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> avgSig2vec;

  double sse, pre, beta, x, sig2, height, sig2Sum=0;
  int nAtoms, i, j;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    // Reset sigma2 sum to zero
    sig2Sum = 0;
    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );

      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
        if( j == tempInd ){
	  // if this is the desired temperature
	  sig2Sum += sig2;
	}
      }
    }
    if( nAtoms == 0 )
      avgSig2vec.push_back( 0.0 );
    else
      avgSig2vec.push_back(sig2Sum / nAtoms);
    nSamps++;
  }
  
  // Convert to a doubles array
  Array1D<double> avgSig2Arr(nSamps);
  for(i=0; i<nSamps; i++)
    avgSig2Arr[i] = avgSig2vec[i];

  fclose(FLfile);
  fclose(VLfile);

  return(avgSig2Arr);
}

/**************************************************************************
 **  Get the variance (sigma^2) of a single kernel chosen at random,
 **  for each iteration of the MCMC.
 **/  
Array1D<double> LevyInfer::randSig2( int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> randSig2vec;
  Array1D<double> equalProbs;

  double sse, pre, beta, x, sig2, height;
  int nAtoms, i, j, randInd=0;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );
  
      // Sample an atom 
      if( j == tempInd ){
        equalProbs = Array1D<double> (nAtoms, 1/(double)nAtoms);
        randInd = rdiscrete(equalProbs);
      }
      for(i=0; i<nAtoms; i++){
      
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
        if( ( randInd == i ) && ( j == tempInd ) )
	  randSig2vec.push_back( sig2 );
      }
    }
    nSamps++;
  }
  
  // Convert to a doubles array
  Array1D<double> randSig2Arr(nSamps);
  for(i=0; i<nSamps; i++)
    randSig2Arr[i] = randSig2vec[i];

  fclose(FLfile);
  fclose(VLfile);

  return(randSig2Arr);
}

/**************************************************************************
 **  Get the magnitude (z) of a single kernel chosen at random,
 **  for each iteration of the MCMC.
 **/  
Array1D<double> LevyInfer::randZ( int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> randZvec;
  Array1D<double> equalProbs;

  double sse, pre, beta, x, sig2, height;
  int nAtoms, i, j, randInd=0;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );
  
      // Sample an atom 
      if( j == tempInd ){
        equalProbs = Array1D<double> (nAtoms, 1/(double)nAtoms);
        randInd = rdiscrete(equalProbs);
      }
      for(i=0; i<nAtoms; i++){
      
        if( getAtom( VLfile, &x, &sig2, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
        if((  randInd == i ) && ( j == tempInd ))
	  randZvec.push_back( height );
      }
    }
    nSamps++;
  }
  
  // Convert to a doubles array
  Array1D<double> randZArr(nSamps);
  for(i=0; i<nSamps; i++)
    randZArr[i] = randZvec[i];

  fclose(FLfile);
  fclose(VLfile);

  return(randZArr);
}

/**************************************************************************
 **  Get the lrf for sample number sampNum
 **/  
void LevyInfer::getLrfSamp
( int sampNum, Array1D<double> &xlocs, 
  Array1D<double> &sig2,		    
  Array1D<double> &heights,
  int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> zsum;

  double sse, pre, beta, x, sig2single, height;
  int nAtoms, i, j;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );
      
      if((j == tempInd) && ( nSamps == sampNum ) && (nAtoms != xlocs.dim()))
        MESSAGE "Data in variable length file does not match length of output vectors" ERROR;
      for(i=0; i<nAtoms; i++){
        if( getAtom( VLfile, &x, &sig2single, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
        }
        if( (nSamps == sampNum) && (j == tempInd) ){
	  xlocs[i] = x;
          sig2[i] = sig2single;
	  heights[i] = height;
	}
      }
    }
    nSamps++;
  }
  
  fclose(FLfile);
  fclose(VLfile);

}

/**************************************************************************
 **  Get the number of atoms in the sampNum iteration of the Markov chain.
 **/  
int LevyInfer::getSampNatoms
( int sampNum, int tempInd ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, beta;
  int nAtoms, j;

  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;
  double tempDbl[1];

  while( fread( tempDbl, sizeof(double), 1, FLfile ) == 1 ){
    // the new iteration number has just been read.

    for( j=0; j<nTemps; j++ ){
      // for each temperature
      // get the parameters for this tempered chain
      getSamp( FLfile, &nAtoms, &sse, &pre, &beta );
          
      if( (j == tempInd) && (sampNum == nSamps) )
        return( nAtoms );   
    }
    nSamps++;
  }
  if( sampNum < 0 )   // For a negative value of sampNum, return the num. atoms 
    return( nAtoms );  // for the last sample of the chain.
  else
    MESSAGE "Requested info. for an invalid sample index" ERROR;  
  fclose(FLfile);
  fclose(VLfile);
  return( -1 );
}

