#ifndef LEVYPROC_H
#define LEVYPROC_H

#include "tnt_array1d.h"
#include "tnt_array1d_utils.h"
#include "moreArray1dUtils.h"
#include "timeKeeper.h"
#include <vector>
#include <S.h>

using TNT::Array1D;
using std::vector;

class LevyProc{
 private:
  double *xlim_;          // A length-2 vector indicating the
			  // boundaries of the region.
  double area;	           // The area of the region.
  Array1D<double> pM_;     // The vector of probabilities for the
			   // multinomial prior on M
  double agam_, rgam_;     // The parameters of the Levy process.
  double asigma_, rsigma_;  // The parameters of the gamma prior for 
                           // the precision of the normal kernels.
  int JmpMax_;	     	   // The maximum number of atoms
  bool symm_;              // Whether the Levy process is symmetric
			   // gamma (otherwise it is gamma)
  double sigmaConst,       // Constants for calculation of the 
    gamConst;              // birth density
  timeKeeper tk;

  vector<double> xLoc_,    // xLoc_, atomHeight_, and atomVar_ are
    atomHeight_,	   // the length-nAtoms_ vectors of
    atomVar_;              // the atom locations, heights, and
			   // variance parameters.  These are stored
			   // as vectors because vectors can easily
			   // have elements added or deleted.
  int nAtoms_;             // The number of atoms.

  /**************************************************************************
   **  Draw a location uniformly from the rectangular region.  Put the coords 
   **  in x.
   **
   **/
  void drawUnifLoc(double *x);

  /**************************************************************************
  **  Obtain the distance between the location of a particular atom
  **  and the smaller of: the location of the next atom to the right,
  **  or the righthand boundary of the region.  Used in proposing an
  **  atom split.  ind is the index of the atom of interest.
  **/
  double distAboveAtom(int ind);

  /**************************************************************************
   **  Find the next atom to the right of that with index ind1.  Return
   **  the index of that atom, and store its location, variance, and
   **  height in x, sig2, and z.
   **/  
  int getPointToRight
    ( int ind1, double *x, double *sig2, double *z);

 public:
  LevyProc(  
    double *xlim,       // The boundaries of the region
    const Array1D<double> &pM, // the prob. vector for the
				// multinomial prior on M
    double agam = 1.0,         // Parameters of the prior for the num. of atoms.
    double rgam = 1.0,         // and the size of each atom
    double asigma = 1.0,       // Parameters of the prior for the precisions
    double rsigma = 1.0,       // sigma^-2 of the kernels.
    bool symm = true           // Whether the Levy process is
			       // symmetric gamma (vs. just gamma)
    );

  ~LevyProc();


  /**************************************************************************
   **  Calculate the summed squared distance of z from the height of the Levy
   **  process surface at the location x, on the log scale.
   **/
  double sumSquaredError( const Array1D<double> &x, 
			  const Array1D<double> &z,
			  double beta = 0.0,
                          bool logScale = false );

  /**************************************************************************
   **  Calculate the summed distance of z from the height of the Levy
   **  process surface at the locations x, on the log scale.
   **/
  double sumError( const Array1D<double> &x, 
		   const Array1D<double> &z,
		   double beta = 0.0,
                   bool logScale = false );

  /**************************************************************************
   **  Draw from the Gamma random field prior; the atom magnitudes
   **  are sampled from their prior dist'n.
   **/
   void drawFromPrior();

  /**************************************************************************
   **  Draw a new atom for the Levy process.  Draw the location x
   **  uniformly from the rectangular region, and draw the height from
   **  its prior, then multiply the height by s.  If symm_ = true,
   **  then the height can be either positive or negative; otherwise
   **  it is always positive.  Sample the variance of the atom from
   **  its inverse gamma prior.  Calculate the pdf (if ln = false) or
   **  the log-pdf (if ln = true) of the birth distribution.  Do not
   **  add the atom to the levy process.
   **/  
  void drawPoint( double *x, double *sig2, double *z, double s,
                 double *pdf, bool ln = true );

  /**************************************************************************
   **  Split an atom in the Levy process.  The index of the atom to be
   **  split must be specified in "ind".  The proposal scale
   **  (delta_gamma) for the magnitude step must be specified in s.  The
   **  parameters of the specified atom are returned in x, sig2, and
   **  z.  The parameters of the new atoms are returned in x_1,
   **  sig2_1, z_1, x_2, sig2_2, and z_2.  The proposal density is
   **  returned in pdf, on the log scale if ln == true; by "proposal
   **  density" we mean the density of the normal proposal for the
   **  magnitude step, times the density of the IG proposal for
   **  sig2_2, times the density of the uniform proposal for x_2.
   **/  
  void splitPoint( int ind, double s, double *x, double *sig2, double *z, 
		   double *x_1, double *sig2_1, double *z_1,
		   double *x_2, double *sig2_2, double *z_2,   
                   double *pdf, bool ln = true );

  /**************************************************************************
   ** Calculate the density of the atom split proposal distribution
   **/  
  double dSplit 
    (int ind, // the index of the split atom
     double x, // The location of the split atom
     double z_1,  // The height of the first new atom
     double sig2_2, double z_2,  // The variance and
  				    // height of the second new atom
     double s,      // the proposal scale for the magnitude step (delta_gamma)
     bool ln = true        // Whether to return the log pdf or the pdf
     );

  /**************************************************************************
   **  Merge an atom in the Levy process with the next atom to its
   **  right.  If there is no atom to the right, return -1 (otherwise
   **  return 0).  The index of the atom to be merged must be
   **  specified in "ind1".  The proposal scale (delta_gamma) for the
   **  magnitude step of the opposite split must be specified in s.
   **  The parameters of the specified atom and the next atom to its
   **  right are returned in x_1, sig2_1, z_1 and x_2, sig2_2, z_2,
   **  respectively.  The index of the next atom to the right is
   **  returned in ind2.  The parameters of the new atom are returned
   **  in x, sig2, z.
   **/  
  int mergePoints( int ind1, int *ind2, double s, 
		    double *x, double *sig2, double *z, 
		    double *x_1, double *sig2_1, double *z_1,
		    double *x_2, double *sig2_2, double *z_2 );
  
  /**************************************************************************
   ** Calculate the density of the atom birth proposal distribution.
   ** s is the scale factor for the atom magnitude proposal; see the
   ** comments for drawPoint().
   **/  
  double dBirth( 
    double x, double sig2, double z,    // The location, variance, and
					// height of the atom
    double s,
    bool ln = true       // Whether to return the log pdf or the pdf
    );

  /**************************************************************************
   **  Evaluate the Levy density \nu/\nu_+ at (z, mu, sigma^2).  This quantity
   **  does not depend on mu.
   **/  
  double dLevy 
  ( double z, double sig2, bool ln = true);

  /**************************************************************************
   **  Add an atom at location x, variance sig2, and height z to the Levy
   **  process so that the atom has index ind in the new list of atoms.
   **  ind should be between -1 and length() inclusive; a value of -1
   **  is taken to be equal to length().  Return the index of the new atom.
   **/  
  int addPoint( double x, double sig2, double z, int ind = -1 );

  /**************************************************************************
   **  Get the location and magnitude of the atom with index ind.
   **/  
  void getPoint( int ind, double *x, double *sig2, double *z );

  /**************************************************************************
   **  Change the atom with index ind to location x, variance sig2,
   **  and height z
   **/  
  void changePoint( int ind, double x, double sig2, double z );

  /**************************************************************************
   **  Delete the atom with index ind.
   **/  
  void deletePoint( int ind );

  /**************************************************************************
   **  The number of atoms.
   **/  
  int length(){ return (nAtoms_); };

  /**************************************************************************
   **  Write the Levy process atom locations and heights out to a
   **  file, appending to the end of the file.
   **/  
  void appendToFile( FILE *outfile );

  /**************************************************************************
   **  Write the Levy process atom locations and heights to stdout
   **/  
  void print();

  /**************************************************************************
   **  Copy the Levy process 
   **/  
  //LevyProc copy();

};

#endif
