/*
*
* Template Numerical Toolkit (TNT)
*
* Mathematical and Computational Sciences Division
* National Institute of Technology,
* Gaithersburg, MD USA
*
*
* This software was developed at the National Institute of Standards and
* Technology (NIST) by employees of the Federal Government in the course
* of their official duties. Pursuant to title 17 Section 105 of the
* United States Code, this software is not subject to copyright protection
* and is in the public domain. NIST assumes no responsibility whatsoever for
* its use by other parties, and makes no guarantees, expressed or implied,
* about its quality, reliability, or any other characteristic.
*
*/

#ifndef MORE_ARRAY1D_UTILS_H
#define MORE_ARRAY1D_UTILS_H

#include <iostream>
#include <string>
#include "tnt_array1d.h"

namespace TNT
{

  /* Multiply arrays and scalars */
  template <class T>
    inline Array1D<T> operator*(const T &a, const Array1D<T> &B){
    Array1D<T> C = B.copy();
    int len = B.dim();
    for (int i=0; i<len;i++){
      C[i] *= a;
    }
    return(C);
  }
  template <class T>
    inline Array1D<T> operator*(const Array1D<T> &A, const T &b){
    return (b*A);
  }  

  /* Divide a scalar by each element of an array. */
  Array1D<double> operator/(double b, const Array1D<double> &A);

  /* Divide each element of an array by a scalar. */
  Array1D<double> operator/(const Array1D<double> &A, double b);

  /* Exponentiation of an array to a power. */
  Array1D<double> operator^(const Array1D<double> &A, double b);

  /* Add a constant to an array */
  Array1D<double> operator+(const Array1D<double> &A, double b);

  /* Subtraction of arrays and constants */
  /*  Array1D<double> operator-(const Array1D<double> &A, const Array1D<double> &B);*/
  Array1D<double> operator-(const Array1D<double> &A, double b);
  Array1D<double> operator-(double a, const Array1D<double> &B);

  /* Comparison of arrays and constants */
  Array1D<int> operator==(const Array1D<int> &A, int b);

  /* exponentiate each element of A */
  Array1D<double> exp(const Array1D<double> &A);

  /* take the absolute value of each element of A */
  Array1D<double> abs(const Array1D<double> &A);

  /* the subarray defined by the elements of A specified by the
     given indicator vector */
  Array1D<double> subarray (const Array1D<double> &A, 
			    const Array1D<int> &elemIndic );

  /* Whether two scalars are approximately equal.  This is defined to
     mean that they are the same to within percentDiffAllowed.  The
     percent difference is found using the ratio of val1 to val2 and
     comparing to 0.  Therefore it matters which value is listed first,
     although for small epsilon it is not likely to change the result.
     If val2 is zero then it is tested whether val2 is within
     percentDiffAllowed of 0. */
  bool approxEqual (double val1, double val2,
			 double percentDiffAllowed);

  /* Tests whether two arrays are approximately equal.  Approximate
     equality is defined to mean that each element of A is approximately
     equal to the corresponding element of B, as evaluated by
     approxEqual(double, double, double) */
  bool approxEqual (const Array1D<double> &A, const Array1D<double> &B, 
		    double percentDiffAllowed);

  /* Tests whether every element of an array is approximately equal to
     a scalar.  Approximate equality is as evaluated by
     approxEqual(double, double, double) */
  bool approxEqual (const Array1D<double> &A, double value,
		    double percentDiffAllowed);

  /* concatenate two arrays */
  Array1D<double>  concat(const Array1D<double> &A, const Array1D<double> &B);

  /* find the unique elements of an array */
  template <class T>
  inline Array1D<T> unique (const Array1D<T> &A){

    int len = A.dim();

    // find the number of unique elements of A
    int nUnique = 0;
    for (int i=0; i<len; i++){
      if (find(A, A[i]) >= i)
	nUnique++;
    }

    // Create an array of doubles containing the unique
    // elements
    Array1D<T> uniqueElems (nUnique);
    int uniqueIndex = 0;
    for (int i=0; i<len; i++){
      if (find(A, A[i]) >= i){
	uniqueElems[uniqueIndex] = A[i];
	uniqueIndex++;
      }
    }
  
    return(uniqueElems);	
  }


  /* get the sum of an array.  If the array is null, return 0. */
  double  sum(const Array1D<double> &A, int start=0, int end=-1);
  int  sum(const Array1D<int> &A, int start=0, int end=-1);

  /* get the product of the elements of an array.  If the array is 
     null, return 1. */
  double  prod(const Array1D<double> &A);

  /* get the max value in an array, assuming that the array is made up
     of positive elements.  Assigns the index of that value
     to maxValueIndex.  If the array is null, returns 0 and sets
     maxValueIndex to -1. */
  double  max(const Array1D<double> &A, int &maxValueIndex);

  /* get the L2 norm of an array.  If the array is null, return 0. */
  double  normL2(const Array1D<double> &A);

  /* Sort the array in place. If decending=T, sort in decending
     order.  */
  void sort(Array1D<double> &A, bool descending=true);
   
  /* The index of the first occurrence of b in A, up to a difference of
     diff (as defined for the function approxEqual).  If b is not found, 
     find returns -1.
  */
  int find(const Array1D<double> &A, double b, double diff = 0.00001);
  int find(const Array1D<int> &A, int b);

  /* the vector of length numValues, with elements starting at start and with last element
     equal to end, equally spaced. */
  Array1D<double> range(double start, double end, int numValues);

  /* Create a vector of values, given a minimum, maximum, and the number
     of values desired.  Space the values equally on the log scale. */
  Array1D<double> rangeInverseScale (double minimum, double maximum, 
				     int numValues);

  /* Create a vector of values, given a minimum, maximum, and the number
     of values desired.  Space the values equally on the log scale. */
  Array1D<double> rangeLogScale (double minimum, double maximum, 
				 int numValues);

  /* Create a vector of values, given a minimum, maximum, and the number
     of values desired.  Space the values equally on the log 1/value
     scale. */
  Array1D<double> rangeInverseLogScale (double minimum, double maximum, 
     int numValues);


  // Write to a file in R/S format
  void writeArray(const Array1D <double> &array, std::string filename);
  void writeArray(const Array1D <double> &array, std::ostream &out);

  // Read from a file in R/S format
  Array1D<double> readDoubleArray(std::string filename);
  Array1D<double> readDoubleArray(std::istream &in);
  Array1D<int> readIntArray(std::string filename);
  Array1D<int> readIntArray(std::istream &in);

  // Reverse the order of the elements
  void reverse(Array1D <double> &array);

  // Calculate the empirical mean of a vector
  double mean( const Array1D<double> &vec );

  // Calculate the empirical variance of a vector
  double var( const Array1D<double> &vec );

  // Calculate the log of the elements of a vector
  Array1D<double> log( const Array1D<double> &vec );

} // namespace TNT

#endif
