# R code (NOT S-PLUS code) for obtaining prior constants for HARK


priorConstsTau2 <- function( signals, varInflate = 2, sp = -1, k ){
  # if sp <= 0 then the smoothing parameter is chosen using REML

  # apply penalized splines and obtain the residual standard deviations
  nPatients <- dim( signals )[1]
  maxTime <- dim( signals )[2]
  mseSig <- rep( 0, nPatients )
  for(i in (1:nPatients)){
    # remove the missing values
    noMiss <- !is.na( signals[i,] )
    zNoMiss <- signals[i,noMiss]
    xNoMiss <- (1:maxTime)[noMiss]
    # obtain the penalized spline estimate
    k = min( k, length( xNoMiss ) )
    if( sp <= 0 )
      visitSmooth <- gam(zNoMiss ~ s(xNoMiss, k=k), method = "REML" )
    else
      visitSmooth <- gam(zNoMiss ~ s(xNoMiss, k=k), sp = sp )
    # calculate  mean squared error
    resids <- zNoMiss - as.vector(visitSmooth$fitted.values)
    mseSig[i] <- sum( resids^2 ) / ( length(resids) - 1 )
  }
hist(sqrt(mseSig))
  m = mean(mseSig)
  if( nPatients == 1 )
    v = 1
  else
    v = varInflate * var(mseSig)

  # find the shape and scale parameters for an inverse gamma prior
  # for the residual variance (i.e. the shape and rate parameters
  # for the gamma prior for the residual precision)
  alpha = m^2/v + 2
  beta = m*(alpha-1)

  return( list(alpha = alpha, beta = beta) )
}

priorConstsBeta0 <- function( signals, thisSigInd, sdDeflate = 10 ){

   mbet <- mean( signals[ thisSigInd, ], na.rm = T );

   nPatients = dim(signals)[1]
   sdY <- rep(0, nPatients)
   for( i in 1:nPatients ){
     thisVisit <- signals[i,]
     sdY[i] <- sd(thisVisit, na.rm = T)
   }
   sbet <- mean(sdY) / sdDeflate

   return( list( mbet=mbet, sbet=sbet ) )
 }

priorConstsBeta0.2 <- function( signals, thisSigInd, sdDeflate = 1 ){

  mbet <- mean( signals[ thisSigInd, ], na.rm = T );

  nPatients = dim(signals)[1]
  meanY <- rep(0, nPatients)
  for( i in 1:nPatients ){
    thisVisit <- signals[i,]
    meanY[i] <- mean(thisVisit, na.rm = T)
  }
  # remove outliers
  rem <- abs( meanY - mean( meanY ) ) / sd( meanY ) > 3
  meanY <- meanY[!rem]
  sbet <- sd(meanY) / sdDeflate

  hist(meanY)
  return( list( mbet=mbet, sbet=sbet ) )
}

priorConstsGamma <- function( signals, k, sp = -1, eps = 0.2 ){

  # apply penalized splines and obtain the empirical distribution of
  # absolute peak heights
  nPatients <- dim( signals )[1]
  peakHeights <- c()
  for(i in (1:nPatients)){
    res = empirRep( signals, i, k=k, sp = sp, eps =eps )
    if( !is.null( res$peakHeights ) )
      # if there is at least one peak
      peakHeights <- c( peakHeights, abs( res$peakHeights ) )
  }
  hist(peakHeights)
  m <- mean( peakHeights )
  v <- var( peakHeights )

  rgam <- m/v
  agam <- rgam * m

  return( list( rgam=rgam, agam=agam ) )
}

priorConstsSigma2 <- function( signals, k, sp = -1, eps = 0.2, timeInflation = 1 ){

  # apply penalized splines and obtain the empirical distribution of
  # peak widths
  nPatients <- dim( signals )[1]
  maxTime <- dim( signals )[2]
  sigma2s <- c()
  for(i in (1:nPatients)){
    res = empirRep( signals, i, k=k, sp = sp, eps =eps )
    if( !is.null( res$sigma2s ) )
      # if there is at least one peak
      sigma2s <- c( sigma2s, res$sigma2s )
  }
  sigma2s <- sigma2s * timeInflation^2
  hist(sigma2s)
  m <- mean( sigma2s )
  v <- var( sigma2s )

  asig <- m^2/v + 2
  rsig <- m*(asig-1) 

  return( list( rsigma=rsig, asigma=asig ) )
}

estimateSigma2 <- function( sigEst, xNoMiss, minMaxLoc, isMax, sigMean ){
# Given the location of a local max / min of an estimated function,
# return an estimate of the variance of the peak associated with
# that max / min.  Do this by finding the first intersection with
# the mean line that is above the max / min, and the first inter-
# section with the mean line that is below the max / min.  The
# difference between these two quantities is roughly four times
# the standard deviation of the peak.

  m = sigMean
  
  # deal with the maximum and minimum cases separately
  if( isMax ){
    # Find the first intersection with the mean line
    # that is below the maximum.
    lb = minMaxLoc
    while( ( lb > 1 ) &&
           ( sigEst[lb] > m ) ){
      lb = lb - 1;
    }
    # Find the first intersection with the mean line
    # that is above the maximum.
    ub = minMaxLoc
    while( ( ub < length(sigEst) ) &&
           ( sigEst[ub] > m ) ){
      ub = ub + 1;
    }
  } else {
    # Find the first intersection with the mean line
    # that is below the maximum.
    lb = minMaxLoc
    while( ( lb > 1 ) &&
           ( sigEst[lb] < m ) ){
      lb = lb - 1;
    }
    # Find the first intersection with the mean line
    # that is above the maximum.
    ub = minMaxLoc
    while( ( ub < length(sigEst) ) &&
           ( sigEst[ub] < m ) ){
      ub = ub + 1;
    }
  }
  # obtain the distance between the upper and lower
  # bounds of the peak, in terms of time units.
  sig = xNoMiss[ub] - xNoMiss[lb]
  # the distance between upper and lower bounds of the
  # peak is about 4 SDs
  sig = sig / 4
  # square to get the variance
  sig2 = sig^2
  return( sig2 )
}

priorConstsEM <- function( signals, k, sp = -1, eps = 0.2 ){

  # apply penalized splines and obtain the empirical distribution of
  # absolute peak heights
  nPatients <- dim( signals )[1]
  maxTime <- dim( signals )[2]
  M <- rep(0, nPatients)
  for(i in (1:nPatients)){
    res = empirRep( signals, i, k=k, sp = sp, eps =eps )
    M[i] = length(res$peakLocs)
  }
  hist(M)
  print(M)

  pM = rep(0, max(M)+1)
  for(i in 1:(max(M)+1))
    pM[i] = sum( M == (i-1) )

  # check for zeros in between nonzero values--this would cause a reducible
  # Markov chain
  nonZero = F
  pMlen = length(pM)
  for( i in (1:pMlen) ){
    if( pM[i] != 0 )
      nonZero = T
    if( nonZero && (pM[i] == 0) )
      pM[i] = 1
  }
  
  # normalize
  pM = pM / sum(pM)
  pM = c( pM, 0 )  # add a zero on the end so that proposed moves that increase pM
                   # past the max are rejected.

  return( pM )
}



empirRep <- function(signals, i, k = dim( signals )[2], sp = -1, eps =0.2,
                     plotER = F){
  # if sp <= 0 then the smoothing parameter is chosen using REML
  
  maxTime <- dim( signals )[2]
  # remove the missing values
  noMiss <- !is.na( signals[i,] )
  zNoMiss <- signals[i,noMiss]
  xNoMiss <- (1:maxTime)[noMiss]

  # obtain the mean value
  sigMean <- mean( zNoMiss )
    
  # obtain the penalized spline estimate
  k = min( k, length( xNoMiss ) )
  if( sp <= 0 )
    visitSmooth <- gam(zNoMiss ~ s(xNoMiss, k=k), method = "REML" )
  else
    visitSmooth <- gam(zNoMiss ~ s(xNoMiss, k=k), sp = sp )
  smoothPred = predict(visitSmooth, newdata=list( xNoMiss=(1:maxTime) ) )
  if( plotER ){
    plot((1:maxTime),smoothPred, type = "l", ylim = range(zNoMiss))
    points( xNoMiss, zNoMiss )
  }

  # calculate  mean squared error
  resids <- zNoMiss - as.vector(visitSmooth$y)
  mseSig <- sum( resids^2 ) / ( length(resids) - 1 )

  # obtain the local maxima
  isMax <- smoothPred[2:(maxTime-1)] > smoothPred[1:(maxTime-2)]
  isMax <- isMax & ( smoothPred[2:(maxTime-1)] > smoothPred[3:maxTime] )
  isMax <- c(F, isMax, F)   # want this vector to have length maxTime
  maxLocs <- which( isMax )
  # obtain the local minima
  isMin <- smoothPred[2:(maxTime-1)] < smoothPred[1:(maxTime-2)]
  isMin <- isMin & ( smoothPred[2:(maxTime-1)] < smoothPred[3:maxTime] )
  isMin <- c(F, isMin, F)   # want this vector to have length maxTime
  minLocs <- which( isMin )
  minMaxLocs <- sort( c( minLocs, maxLocs ) )
  # obtain the height of the function at the mins and maxs
  heights <- smoothPred[ minMaxLocs ]

  # for each minimum / maximum,
  nMinMax <- length( heights )
  peakLocs <- c()
  peakHeights <- c()
  sigma2s <- c()
  if( nMinMax > 0 ){
    for( j in (1:nMinMax) ){
      # if the height is more than eps different from the last height
      if( (j==1) || ( abs( heights[j] - heights[j-1] ) > eps ) ){
        # and if it is a minimum and below the mean, or
        # if it is a minimum and above the mean
        if( ( ( isMin[ minMaxLocs[j] ] ) && ( smoothPred[ minMaxLocs[j] ] < (sigMean-eps) ) ) ||
           ( ( isMax[ minMaxLocs[j] ] ) && ( smoothPred[ minMaxLocs[j] ] > (sigMean+eps) ) ) ){
           # add this peak height to the vector of peak heights.
          peakLocs <- c( peakLocs, minMaxLocs[j] )
          peakHeights <- c(peakHeights,
                           smoothPred[ minMaxLocs[j] ] - sigMean )
          sigma2s <- c( sigma2s,
                       estimateSigma2( smoothPred, (1:maxTime), minMaxLocs[j],
                                      isMax[ minMaxLocs[j] ], sigMean ) )
        
        }
      }
    }
  }

  return( list( beta0 = sigMean, peakLocs = peakLocs, peakHeights = peakHeights, sigma2s = sigma2s,
               mse=mseSig, sp = visitSmooth$sp, df = sum(visitSmooth$edf) ) )
}

estFun <- function( beta0, peakLocs, peakHeights, sigma2s, maxTime ){

  fun <- rep( beta0, maxTime )
  nAtoms <- length( peakLocs )
  for( i in 1:nAtoms ){
    fun <- fun + peakHeights[i] * exp(-( (1:maxTime)-peakLocs[i] )^2 / (2*sigma2s[i]) )
  }
  return( list(x=1:maxTime, y=fun) )
}

