
# fits HARK to the simulated data described in Woodard,
# Crainiceanu, and Ruppert (2012).

remove(list=ls())
set.seed(451)
m = 100  # the signal length
n = 500   # the number of signals to simulate
x = (1:m)   # the times for the observations
y = matrix(1,nrow=n,ncol=m)
# sample the random location of the spike
u = runif(n,.03,.87) * m
# the dip is just after the spike
u2 = u + (.1*m)

# define the response vector
response <- runif(n=n, 10, 20)

trueSig = matrix(1,nrow=n,ncol=m)
for (i in 1:n)
{ # for each subject
  # calculate the mean function
  trueSig[i,] = response[i] * exp( -(x-u[i])^2/(2*5) ) - response[i] * exp( -(x-u2[i])^2/(2*5) )
  # sample the signal 
  y[i,] = trueSig[i,] + rnorm(m,mean=0,sd=sqrt(1))
}

response = response + rnorm(n, mean=0, sd=5)

# specify the subset of patients for which HARK will
# be applied
patientSubset <- c( 1:50 )
datSubset = y[ patientSubset, ]
nPats = dim(datSubset)[1]

# save to a file
write.table( data = datSubset, file = "simulDat.txt", sep = ",",
            dimnames.write = F)

################  HERE, RUN THE CODE IN simulHARKpriorConsts.R in R
################  BEFORE CONTINUING

# get the HARK hyperparameters
hyper = as.vector( as.matrix(
  read.table( file = "hyperParams.txt", sep = ",", header = F)))
apre = hyper[1]
rpre = hyper[2]
agam = hyper[3]
rgam = hyper[4]
asigma = hyper[5]
rsigma = hyper[6]
pM = as.vector( as.matrix(
  read.table(file = "pM.txt", sep = ",", header = F)))


library(hark)
library(coda)
# get the code for consistent batch means
source("batchmeans.R")




# Run MCMC
ItnMax = 1e3 #1e7 
ItnBrn = 1e3 #5e6 


rootName = "sig"

for( i in 1:nPats ){
  oneSignal = data.frame( x = (1:m),
                        z = datSubset[i,] )

  res = priorConstsBeta0( datSubset, thisSigInd=i,
    sdDeflate = 10 )
  mbet = res$mbet
  sbet = res$sbet

  fileName = paste( rootName, i, sep="" )

  lrfRun(Dat = oneSignal, Bdy = BBox( c(0,m), expand = .2 ),
    ItnMax = ItnMax, ItnBrn = ItnBrn,
    SavMax = 1e4,
    PriPar  = list(apre = apre, rpre = rpre,                   
                   asigma = asigma, rsigma = rsigma,
                   mbet = mbet, sbet = sbet,
                   pM = pM, agam = agam,   
                   rgam = rgam),
    OutFile=fileName,
    dX = c( dnew=2, dmvmag=0.4, dmvsc=0.7, dmvloc=15, dspl=0.5 ),
    pX = c(pre=.02,bet=.02,add=.13,loc=.09,mag=.11,chsc=.09,rem=.14, spl=0.20, mer=0.20),      #  Move Prob's
    symm = T, printStats = T, temps= c(1,2))

  thisMCMC <- getMcmc( fileName,
        coda.loc = "/home/fs01/dbw59/rlibs", getAvgZ = T,
        getAvgSig2 = T)

  # Effective sample size computation
  minESS <- min(effectiveSize(thisMCMC[,c(1:4,9:10)]))

  if( minESS < 100 ){
    print(paste("ESS vector for signal ", i, " is: " ))
    print(effectiveSize(thisMCMC[,c(1:4,9:10)]))
  }

  # Geweke diagnostic computation
  thisGew <- geweke.diag(thisMCMC[,c(1:4,9:10)])$z

  if( any( abs(thisGew) > 3.53, na.rm=T ) ){  # this is Bonferroni-adjusted, assuming 20 subjects (a bit conservative).
    print(paste("Geweke diagnostic vector for signal ", i, " is: " ))
    print( thisGew )
  }

  # Monte Carlo SE computation
  # Do not calculate the ratio SE/est for \beta_{0i}, since the estimate
  # is so close to zero that this ratio is typically large (the SEs are
  # small however).
  thisSE <- bmmat( thisMCMC[,c(1,2,4,9:10)] )[,"se"]
  thisEst <- colMeans( thisMCMC[,c(1,2,4,9:10)] )

  if( any( thisSE / thisEst > 0.01 ) ){
    print(paste("Monte Carlo SE vector for signal ", i, " is: " ))
    print(thisSE)
    print(paste("Monte Carlo estimate vector for signal ", i, " is: " ))
    print(thisEst)
    print("Ratio is: ")
    print(thisSE / thisEst)
  }
}

# check convergence
i=1
fileName = paste( rootName, i, sep="" )
thisMCMC <- getMcmc( fileName, getAvgZ = T,
        getAvgSig2 = T)

summary(thisMCMC)
par(mfrow = c(6,1))
traceplot(thisMCMC[,1:6])
traceplot(thisMCMC[,5:8])
traceplot(thisMCMC[,9:10])
min(effectiveSize(thisMCMC))
par(mfrow = c(1,1))
crosscorr.plot(thisMCMC)

sampNum=8000
par(mfrow=c(1,1))
sampPlot(fileName, sampNum = sampNum, nxy = 100, ylim = c(-20,20),
         xlim=c(-10,110))
larkPlot(fileName, sampNum = sampNum, add = T)
points(datSubset[i,])  


## regress the outcome on the predictor functions (2nd stage)
initPredsInd = c(1, rep(0, 6*5) )
predPenalty = c( 0.2, 0.2, 0.2, 1, 0.2, 0.2 )
harkRegress( response = response[patientSubset], rootName,
            outFile = paste( rootName, "Regress2.bin", sep="" ),
            initPredsInd = initPredsInd,  nIter = 20000,
            priorVar = c(10, rep(1, 6*5)), priorType = 0,
            priorInMcmc = F,
            predPenalty = predPenalty )

regMcmc <- getRegressMcmc(
      McmcFile = paste( rootName, "Regress2.bin", sep="" ),
      nCoefs = 31 ) 
summary(regMcmc)

regFns <- getRegressFns(
      McmcFile = paste( rootName, "Regress2.bin", sep="" ),
      nCoefs = 31, nGrid = 100,
      model = c(0,0,0,1,0,0),
      predPenalty = predPenalty )

# calculate error
par(mfrow = c(3,2))
for( j in 1:6 ){
  lowerBd = regFns$predBounds[j,1]
  upperBd = regFns$predBounds[j,2]
  thisLocs = regFns$predLocs[[j]]
  thisVals = regFns$predFn[[j]]
  thisLower = regFns$predLower[[j]]
  thisUpper = regFns$predUpper[[j]]
  insideBds = (thisLocs >= lowerBd) & (thisLocs <= upperBd) 
  predVals = thisVals[insideBds]
  predLocs = thisLocs[insideBds]
  lowerVals = thisLower[insideBds]
  upperVals = thisUpper[insideBds]
  yvals = c(predVals, lowerVals, upperVals )
  ylim = c( min( yvals ), max( yvals ) )
  plot( x = predLocs, y = predVals, type = "l",
       xlim = c(lowerBd, upperBd), ylim = ylim, col = 1)
  lines( x = predLocs, y = lowerVals,
        lty = 2 )
  lines( x = predLocs, y = upperVals,
        lty = 2 )
  if( j==4 ){
    trueVals = thisLocs[insideBds]
    lines( x= predLocs, y = trueVals, lty = 4 )
  } 
}
lowerBd = regFns$predBounds[4,1]
upperBd = regFns$predBounds[4,2]
thisLocs = regFns$predLocs[[4]]
thisVals = regFns$predFn[[4]]
thisLower = regFns$predLower[[4]]
thisUpper = regFns$predUpper[[4]]
insideBds = (thisLocs >= lowerBd) & (thisLocs <= upperBd) 
predVals = thisVals[insideBds]
predLocs = thisLocs[insideBds]
lowerVals = thisLower[insideBds]
upperVals = thisUpper[insideBds]
trueVals = thisLocs[insideBds]
nPredLocs = length(predLocs)
sqrErr = (predVals - trueVals)^2
RMSE = sqrt( sum(sqrErr) / nPredLocs )
MAE = mean( abs( predVals - trueVals ) )
bias = mean(predVals - trueVals)
mean( upperVals - lowerVals )
mean( (trueVals >= lowerVals) & (trueVals <= upperVals) )
RMSE
MAE

# convergence diagnosis
traceplot(regMcmc[,26])
plot(as.numeric(regMcmc[,15]!=0), type = "l", col = 1)
regESS = effectiveSize(regMcmc)
for(i in (3:31) )
  regESS = c(regESS, effectiveSize(as.numeric(regMcmc[,i]==0)))
min( regESS[regESS != 0])

par(mfrow=c(1,1))
crosscorr.plot(regMcmc)

# calculate the posterior mode model
postModeModel( McmcFile = paste( rootName, "Regress2.bin", sep="" ),
              nCoefs = 31, priorInMcmc = F, predPenalty = predPenalty )

# calculate the marginal inclusion probabilities of the covariates
margPredProbs( McmcFile = paste( rootName, "Regress2.bin", sep="" ),
              nCoefs = 31, priorInMcmc = F, predPenalty = predPenalty ) 

